package com.huimv.guowei.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.DuckHealthInfo;
import com.huimv.guowei.admin.mapper.BaseDuckInfoMapper;
import com.huimv.guowei.admin.mapper.DuckHealthInfoMapper;
import com.huimv.guowei.admin.service.IDuckHealthInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class DuckHealthInfoServiceImpl extends ServiceImpl<DuckHealthInfoMapper, DuckHealthInfo> implements IDuckHealthInfoService {

    @Autowired
    private DuckHealthInfoMapper healthInfoMapper;
    @Autowired
    private BaseDuckInfoMapper infoMapper;

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String duckNum = paramsMap.get("id");
        String farmId = paramsMap.get("farmId");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");

        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String format = sdf.format(date);
        QueryWrapper<DuckHealthInfo> queryWrapper = new QueryWrapper<>();
        if (startDate == null || "".equals(startDate)) {
            queryWrapper.eq("duck_id", duckNum).eq("farm_id", farmId).orderByAsc("health_date");
        }else {
            startDate = startDate + " 00:00:00";
            endDate = endDate + " 23:59:59";
            queryWrapper.eq("duck_id", duckNum).eq("farm_id", farmId).between("health_date", startDate, endDate).orderByAsc("health_date");
        }
        Page<DuckHealthInfo> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, healthInfoMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result addHealth(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) throws ParseException {
        String farmId = paramsMap.get("farmId");
        String duckId = paramsMap.get("duckId");
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", duckId);
        BaseDuckInfo baseDuckInfo = infoMapper.selectOne(queryWrapper);
        String date = paramsMap.get("date");
        date = date + " 00:00:00";
        String healthResult = paramsMap.get("healthResult");
        String treatmentMeasure = paramsMap.get("treatmentMeasure");
        DuckHealthInfo healthInfo = new DuckHealthInfo();
        healthInfo.setDuckNum(baseDuckInfo.getDuckNum());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        healthInfo.setFarmId(Integer.parseInt(farmId));
        healthInfo.setHealthDate(sdf.parse(date));
        healthInfo.setHealthResult(healthResult);
        healthInfo.setDuckId(Integer.parseInt(duckId));
        healthInfo.setTreatmentMeasure(treatmentMeasure);
        healthInfo.setUnitId(baseDuckInfo.getUnitId());
        healthInfo.setSymptom(paramsMap.get("symptom"));
        healthInfo.setCure(paramsMap.get("cure"));
        healthInfo.setMedicine(paramsMap.get("medicine"));
        healthInfo.setMeasure(paramsMap.get("measure"));
        healthInfoMapper.insert(healthInfo);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listByH5(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<DuckHealthInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("health_date").last(" limit 10");
        return new Result(ResultCode.SUCCESS, healthInfoMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<DuckHealthInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq(StringUtils.isNotBlank("unit_id"),"unit_id", unitId)
                .orderByDesc("health_date");;
        Page<DuckHealthInfo> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, healthInfoMapper.selectPage(page, queryWrapper));
    }


}
