package com.huimv.guowei.admin.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.EnvRegularCallEgg;
import com.huimv.guowei.admin.entity.EnvRegularCallFeeding;
import com.huimv.guowei.admin.mapper.BaseDuckInfoMapper;
import com.huimv.guowei.admin.mapper.EnvRegularCallEggMapper;
import com.huimv.guowei.admin.mapper.EnvRegularCallFeedingMapper;
import com.huimv.guowei.admin.service.IEnvRegularCallEggService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.utils.Print;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 固定电子秤 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnvRegularCallEggServiceImpl extends ServiceImpl<EnvRegularCallEggMapper, EnvRegularCallEgg> implements IEnvRegularCallEggService {

    @Autowired
    private EnvRegularCallEggMapper eggMapper;
    @Autowired
    private BaseDuckInfoMapper infoMapper;
    @Autowired
    private EnvRegularCallFeedingMapper feedingMapper;

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");//显示类型
        String duckCode = paramsMap.get("id");//鸭子的id
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        if ("1".equals(type)) {
            QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("duck_id", duckCode).eq("farm_id", farmId);
            Page<EnvRegularCallEgg> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, eggMapper.listPage(page, queryWrapper));
        } else {
            String dateType = paramsMap.get("dateType");
            QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId).eq("duck_id", duckCode);
            if ("1".equals(dateType)) {
                Date timesmorning = DataUill.getTimesmorning();
                queryWrapper.ge("call_date", timesmorning);

            } else if ("2".equals(dateType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                queryWrapper.ge("call_date", calendar.getTime());

            } else if ("3".equals(dateType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                queryWrapper.ge("call_date", timesMonthmorning);

            } else {
                String startDate = paramsMap.get("startDate");
                String endDate = paramsMap.get("endDate");
                startDate = startDate + " 00:00:00";
                endDate = endDate + " 23:59:59";
                queryWrapper.between("call_date", startDate, endDate);
            }
            return new Result(ResultCode.SUCCESS, eggMapper.listDay(queryWrapper));
        }
    }

    @Override
    public Result addEgg(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) throws ParseException {
        String farmId = paramsMap.get("farmId");
        String duckId = paramsMap.get("duckId");
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", duckId);
        BaseDuckInfo baseDuckInfo = infoMapper.selectOne(queryWrapper);
        String date = paramsMap.get("date");
        date = date + " 00:00:00";
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String eggNum = paramsMap.get("eggNum");
        String eggWeight = paramsMap.get("eggWeight");
        EnvRegularCallEgg egg = new EnvRegularCallEgg();
        egg.setDuckNum(baseDuckInfo.getDuckNum());
        egg.setFarmId(Integer.parseInt(farmId));
        egg.setDuckWeight(Integer.parseInt(eggWeight));
        egg.setEggNum(Integer.parseInt(eggNum));
        egg.setCallDate(sdf.parse(date));
        egg.setDuckId(Integer.parseInt(duckId));
        eggMapper.insert(egg);
        return new Result(10000,"添加成功!",true);
    }

    @Override
    public Result updateEgg(HttpServletRequest httpServletRequest, EnvRegularCallEgg egg) {
        eggMapper.updateById(egg);
        return new Result(10000, "修改成功！", true);
    }

    @Override
    public Result deleteEgg(HttpServletRequest httpServletRequest, Map<String,String> paramsMap) {
        String id = paramsMap.get("id");//此时这里就是表中的id
        eggMapper.deleteById(id);
        return new Result(10000, "删除成功!", true);
    }

    @Override
    public void printEgg(HttpServletResponse response, Map<String, String> paramsMap) throws Exception {
        String duckCode = paramsMap.get("duckCode");
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("duck_num", duckCode);
        List<EnvRegularCallEgg> eggs = eggMapper.printEggList(queryWrapper);
        QueryWrapper<EnvRegularCallFeeding> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("duck_num", duckCode);
        List<EnvRegularCallFeeding> feedings = feedingMapper.listDay(queryWrapper1);
        Print.printEgg(eggs, feedings);
    }
    @Override
    public Result listByH5(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("call_date").last(" limit 10");
        return new Result(ResultCode.SUCCESS, eggMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq(StringUtils.isNotBlank("unit_id"),"unit_id", unitId)
                .orderByDesc("call_date");
        Page<EnvRegularCallEgg> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, eggMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result listSeven(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
        queryWrapper.eq("farm_id", farmId).eq(StringUtils.isNotBlank("unit_id"),"unit_id", unitId).ge("call_date", calendar.getTime());
        QueryWrapper<EnvRegularCallEgg> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq(StringUtils.isNotBlank("unit_id"), "unit_id", unitId);
        queryWrapper1.select(" IFNULL(SUM(duck_weight),'0') duckWeight, IFNULL(SUM(egg_num),'0') eggNum");
        EnvRegularCallEgg egg = eggMapper.selectOne(queryWrapper1);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("eggNum", egg.getEggNum());
        jsonObject.put("egg", eggMapper.selectList(queryWrapper));
        jsonObject.put("eggWeight", egg.getDuckWeight());
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

}
